%**************************************************************************
%  Author: Tim Chartier
%  Title:  lsFace.m
%  Description: This code creates 2D fractal mountains.
%**************************************************************************
close all

%% Input parameters
targetImage = imread('chartier.jpg');

% Library of images parameters
directoryName = 'Celebrity';
fileType = 'jpg';

%% Load the target image
targetImage = targetImage(:,:,1);   % Strip off first channel in case image saved in several channels.
[nRows,nCols] = size(targetImage);
targetImage = cast(targetImage,'double');
targetImage = reshape(targetImage,nRows*nCols,1);

%% Change mean and std of input image.

% Chosen std and mean. 
% It can be any number that it is close to the std and mean of most of the images.
chosenMean = 100;
chosenStd  = 80;

temp = targetImage;
meanImage= mean(temp);
stdImage = std(temp);
temp = (temp-meanImage)*chosenStd/stdImage + chosenMean;
targetImage = temp;

%% Load the library of "Training" images
d = dir([directoryName,'/*.',fileType]);

numberOfTrainingPhotos = length(d);

trainingMatrix = zeros(nRows*nCols,numberOfTrainingPhotos);
for i=1:numberOfTrainingPhotos
      filename = [directoryName,'/',d(i).name];
      imageTemp = imread(filename);
      imageTemp = imageTemp(:,:,1);
      [r,c] = size(imageTemp);
      if (r ~= nRows) || (c ~= nCols)
         fprintf('**** Training images must be of same size ****\n')
         disp(['**** Mismatched size when reading image ',d(i).name,' ****'])
      end
      trainingMatrix(:,i) = reshape(imageTemp,r*c,1);
end

%% Show the image library

figure('Name','Original library of images')
for i=1:numberOfTrainingPhotos
    subplot(ceil(sqrt(numberOfTrainingPhotos)),ceil(sqrt(numberOfTrainingPhotos)),i)
    image(reshape(trainingMatrix(:,i),nRows,nCols))
    axis equal off
    colormap(gray(256));
    drawnow;
end

%% Here we change the mean and std of all images to reduce the error due to lighting conditions.
for i=1:numberOfTrainingPhotos
    temp = double(trainingMatrix(:,i));
    imageMean = mean(temp);
    imageStd  = std(temp);
    trainingMatrix(:,i) = (temp - imageMean)*chosenStd/imageStd + chosenMean;
end

% Show the altered image library

figure('Name','Altered image library')
for i=1:numberOfTrainingPhotos
    subplot(ceil(sqrt(numberOfTrainingPhotos)),ceil(sqrt(numberOfTrainingPhotos)),i)
    image(reshape(trainingMatrix(:,i),nRows,nCols))
    axis equal off
    colormap(gray(256));
    drawnow;
end

%% Perform least-squares fit

lsSolution = trainingMatrix\targetImage;
lsImage = trainingMatrix*lsSolution;

%% Display original and LS approximation images
figure

subplot(1,2,1) 
image(reshape(targetImage,nRows,nCols))
axis off equal tight

lsImage = reshape(lsImage,nRows,nCols);
subplot(1,2,2)
image(lsImage)
axis off equal tight

colormap(gray(256));

%% Print results
figure
bar(lsSolution)
[lsSorted,iSorted] = sort(lsSolution,'descend');
fprintf('\n\n Weight    Image File\n')
fprintf('================================\n')
for i=1:numberOfTrainingPhotos
    fprintf('%8.5f   %s\n',lsSorted(i),d(iSorted(i)).name);
end

return

%% Output library of images

for i=1:numberOfTrainingPhotos
    tmp = reshape(trainingMatrix(:,i),nRows,nCols);
    filename = sprintf('%02d.jpg',i);
    imwrite(tmp,gray(256),filename,'jpg')
end


